// This is the JavaScript code that makes up the core AdventureCreator engine.
// You should not change this file.  Any code that you need for your game should go in _game.js

var gVariables = "Variables";

var gCurrentPage = "CurrentPage";
var gCurrentLocale = "CurrentLocale";

var gDefDelim = "AdventureCreator";

var gDefStartPage = "_begin";
var gDefStartLoc = "en";

var gLocId = "locId";
var gImageId = "image";
var gHotspotsId = "hotspots";
var gSaysRowId = "saysRow";
var gImageBorderClass = "imgborder";

var anim = {};
var pageExitFcn = null;


//--------------------------------------------------------//
// Functions you can call from your _game.js
//
// Although these functions are used by the game engine, 
// you can call them from your _game.js
//

// Used for moving from one page to the next.
function GotoPage(page)
{
    LoadPage(page);
}

// Used for changing the language (also moves to a new page).
function ChangeLocale(language, page)
{
    LoadLocale(language, function () { LoadPage(page) });
}

// This function sets the variable to a number
function SetVar(name, value)
{
    var strings = ReadStorageString(gVariables).split(':');
    var found = false;
    var varStr = "";

    for (var index in strings)
    {
        var str = strings[index];
        var eqIndex = str.indexOf('=');
        if (eqIndex != -1)
        {
            var to = str.substring(0, eqIndex);
            var toVal = str.substring(eqIndex + 1, str.length);
            if (to == name)
            {
                toVal = value;
                found = true;
            }
            varStr += to + "=" + toVal + ":";
        }
    }

    if (!found)
    {
        varStr += name + "=" + value;
    }
    else
    {
        // Strip off the last ":"
        varStr = varStr.slice(0, varStr.length - 1);
    }

    SetStorageString(gVariables, varStr);
}


// This function reads the variable and returns its value as a number.  
// If the variable doesn't exist or isn't a number, this returns 0.
function ReadVar(name)
{
    var fullName = name + "=";
    var val = 0;
    var strings = ReadStorageString(gVariables).split(':');
    for (var i = 0; i < strings.length; i++)
    {
        var str = strings[i];
        while (str.charAt(0) == ' ')
        {
            str = str.substring(1, str.length);
        }

        if (str.indexOf(fullName) == 0)
        {
            val = parseInt(str.substring(fullName.length, str.length));
            if (isNaN(val))
            {
                val = 0;
            }
            break;
        }
    }
    return val;
}


//--------------------------------------------------------//
// Internal engine functions
//
// These functions are used internally by the game engine.
// You shouldn't need to call them from your _game.js
//

function LoadPage(page)
{
    ClearPage();
    if (typeof (pages[page]) != "undefined")
    {        
        SaveCurrentPageName(page);
        pages[page](pts[page]);
    }
    else
    {
        var element = document.getElementById("toptext");
        element.innerHTML = "The page you tried to navigate to (" + page + ") doesn't exist.";
    }
    PerPageInit();
}


function PerPageInit()
{
    if (typeof (GamePerPageInit) != "undefined")
    {
        GamePerPageInit();
    }
    else
    {
        ShowScore();
        ShowDebug();
        ShowAds();
    }
}

function ShowScore()
{
    if (typeof (GameShowScore) != "undefined")
    {
        GameShowScore();
    }
}

function ShowDebug()
{
    if (typeof (GameShowDebug) != "undefined")
    {
        GameShowDebug();
    }
    else if (typeof (HelperShowDebug) != "undefined")
    {
        HelperShowDebug();
    }
}

function ShowAds()
{
    if (typeof (GameShowAds) != "undefined")
    {
        GameShowAds();
    }
}

function ClearPage()
{
    if (pageExitFcn != null)
    {
        pageExitFcn();
        pageExitFcn = null;
    }

    if (typeof (anim.timerId) != "undefined" && anim.timerId != 0)
    {
        clearInterval(anim.timerId);
        anim.timerId = 0;
    }

    // The hotspots div contains both the image and the added areas,
    // so we need to save away a copy of the image, deleting everything
    // in the hotspots, and then put the image back.
    var hotspots = document.getElementById(gHotspotsId);
    var image = document.getElementById(gImageId);

    hotspots.innerHTML = "";
    hotspots.insertBefore(image, hotspots.firstChild);

    var tr = document.getElementById(gSaysRowId);
    tr.innerHTML = "";
}

function SetStartPage()
{
    var page = gDefStartPage;
    if (typeof (gameStartPage) != "undefined")
    {
        page = gameStartPage;
    }

    SaveCurrentPageName(page);
}

function SaveCurrentPageName(page)
{
    SetStorageString(gCurrentPage, page);
}

function GetCurrentPageName()
{
    var page = ReadStorageString(gCurrentPage);
    return page;
}

function SaveCurrentLocale(loc)
{
    SetStorageString(gCurrentLocale, loc);
}

function GetCurrentLocale()
{
    var loc = ReadStorageString(gCurrentLocale);
    return loc;
}

function LoadCurrentPage()
{
    page = GetCurrentPageName();
    if (page == "" || page == "undefined")
    {
        page = gDefStartPage;
        if (typeof (gameStartPage) != "undefined")
        {
            page = gameStartPage;
        }
    }

    loc = GetCurrentLocale();
    if (loc == "")
    {
        loc = gDefStartLoc;
    }

    ChangeLocale(loc, page);
}


function LoadLocale(loc, callback)
{
    SaveCurrentLocale(loc);

    var curLocScript = document.getElementById(gLocId);
    if (curLocScript != null)
    {
        curLocScript.parentNode.removeChild(curLocScript);
    }

    var file = "files/text." + loc + ".js";

    var head = document.getElementsByTagName('head')[0];
    var script = document.createElement('script');
    script.type = 'text/javascript';
    script.src = file;
    script.id = gLocId;

    script.onreadystatechange = callback;
    script.onload = callback;

    head.appendChild(script);
}



//--------------------------------------------------------//
// Advanced Animation timer
//
// This function is called for animations.  
function AdvAnim(anim)
{
    if (anim.curReverse)
    {
        anim.curPicture--;
        if (anim.curPicture < 0)
        {
            if (anim.pingpong && anim.reverse)
            {
                anim.curPicture = 1;
                anim.curReverse = false;
            }
            else if (anim.loop)
            {
                if (anim.pingpong) //logic says reverse == false
                {
                    anim.curPicture = 1;
                    anim.curReverse = false;
                }
                else
                {
                    anim.curPicture = anim.pictures.length - 1;
                }
            }
            else
            {
                anim.curPicture = 0;
                clearInterval(anim.timerId);
            }
        }
    }
    else
    {
        anim.curPicture++;
        if (anim.curPicture == anim.pictures.length)
        {
            if (anim.pingpong && !anim.reverse)
            {
                anim.curPicture -= 2;
                anim.curReverse = true;
            }
            else if (anim.loop)
            {
                if (anim.pingpong) //logic says reverse == true
                {
                    anim.curPicture -= 2;
                    anim.curReverse = true;
                }
                else
                {
                    anim.curPicture = 0;
                }
            }
            else
            {
                anim.curPicture--;
                clearInterval(anim.timerId);
            }
        }
    }

    document.getElementById(gImageId).src = anim.pictures[anim.curPicture].src;
}


//--------------------------------------------------------//
// Pages functions
//
// These are functions called from pages.js
function SetImage(path, border)
{
    var image = document.getElementById(gImageId);
    image.src = path;

    if (typeof (border) != "undefined" && border == true)
    {
        image.className = gImageBorderClass;
    }
    else
    {
        image.className = "";
    }
}


function AddAnimation(num, timeout, loop, reverse, pingpong)
{
    var animNumPicts = num;
    anim.timeout = timeout;
    anim.loop = loop;
    anim.reverse = reverse;
    anim.pingpong = pingpong;
    anim.curReverse = reverse;

    var image = document.getElementById(gImageId);
    var imagePath = image.src;
    var indexOfDot = imagePath.lastIndexOf(".");
    var imagePathNoExtension = imagePath.substring(0, indexOfDot);
    var imageExtension = imagePath.substring(indexOfDot);

    if (reverse)
    {
        // We're on the last image.  Need to strip off the number to make the array.
        var numToStrip = Math.floor(num / 10) + 1;
        var index = imagePathNoExtension.length - numToStrip;
        imagePathNoExtension = imagePathNoExtension.substring(0, index);
        imagePath = imagePathNoExtension + imageExtension;
    }

    anim.pictures = new Array(animNumPicts);
    anim.pictures[0] = new Image();
    anim.pictures[0].src = imagePath;

    for (var index = 1; index < animNumPicts; index++)
    {
        anim.pictures[index] = new Image();
        anim.pictures[index].src = imagePathNoExtension + index + imageExtension;
    }

    if (reverse)
    {
        anim.curPicture = num - 1;
    }
    else
    {
        anim.curPicture = 0;
    }

    if (anim.timeout > 0)
    {
        anim.timerId = setInterval("AdvAnim(anim)", anim.timeout);
    }
}


var globalAreaId = 0;

function AddArea(x, y, w, h, onclick, href, title, onmousedown, onmouseup, condition)
{
    var areaID;
    var style, sheet;
    var hotspots, div, a, b, p;

    if (typeof (condition) != "undefined" && condition == false)
    {
        return;
    }

    areaID = "area" + globalAreaId++;

    if (x < 0)
    {
        x = 0;
    }

    if (y < 0)
    {
        y = 0;
    }

    if (x + w > 100)
    {
        w = 100 - x;
    }

    if (y + h > 100)
    {
        h = 100 - y;
    }

    style = "position: absolute; left:" + x + "%; top:" + y + "%; width:" + w + "%; height:" + h + "%;"

    sheet = document.createElement("style");
    sheet.type = "text/css";
    sheet.innerHTML = "." + areaID + " { " + style + " }; ";
    document.head.appendChild(sheet);

    hotspots = document.getElementById(gHotspotsId);

    div = document.createElement("div");

    a = document.createElement("a");
    a.className = areaID;
    a.id = areaID;
    a.setAttribute("aria-haspopup", "false");

    if (onclick == "#")
    {
        onclick = "";
    }
    else
    {
        onclick += ";";
    }

    if (href != "#")
    {
        onclick += "LoadPage(\"" + href + "\");";
    }

    if (onclick != "#")
    {
        a.setAttribute("onclick", onclick);
    }

    if (typeof (onmousedown) != "undefined")
    {
        if (onmousedown != "#")
        {
            a.setAttribute("onmousedown", onmousedown);
        }
    }
    if (typeof (onmouseup) != "undefined")
    {
        if (onmouseup != "#")
        {
            a.setAttribute("onmouseup", onmouseup);
        }
    }

    p = document.createElement("p");
    title = ConvertBrackets(title);
    p.innerHTML = title;

    b = document.createElement("b");
    b.className = areaID;

    div.appendChild(a);
    div.appendChild(p);
    div.appendChild(b);

    var image = document.getElementById(gImageId);

    hotspots.insertBefore(div, hotspots.firstChild);
    hotspots.insertBefore(image, hotspots.firstChild);
    hotspots.appendChild(div);
}


function AddSay(id, onclick, href, title)
{
    var tr = document.getElementById(gSaysRowId);
    var td = document.createElement("td");

    td.id = id;

    if (onclick == "#")
    {
        onclick = "";
    }
    else
    {
        onclick += ";";
    }

    if (href != "#")
    {
        onclick += "LoadPage(\"" + href + "\");";
    }

    td.setAttribute("onclick", onclick);

    if (title != "#")
    {
        title = ConvertBrackets(title);
        td.innerHTML = title;
    }

    tr.appendChild(td);
}




//--------------------------------------------------------//
// Storage functions
//
// These functions are used for storing and changing 
// cookies or localStorage in the site.  The variable functions
// use these.

function UseStorage()
{
    var storageDefined = false;

    // Wrap in try catch to work around bugs in Firefox.
    try
    {
        if (typeof (localStorage) != "undefined")
        {
            storageDefined = true;
        }
    }
    catch (ex)
    {

    }

    return storageDefined;
}


// This function creates a cookie or localStorage (if necessary) and sets its (string) value
function SetStorageString(name, value)
{
    delim = gDefDelim;
    if (typeof (gameDelim) != "undefined")
    {
        delim = gameDelim;
    }

    if (UseStorage())
    {
        localStorage.setItem(name + delim, String(value));
    }
    else
    {
        var days = 365; // Number of days to keep the cookies

        var expires = new Date();
        expires.setTime(expires.getTime() + (days * 24 * 60 * 60 * 1000));

        document.cookie = name + delim + "=" + value + "; path=/" + ((expires == null) ? "" : "; expires=" + expires.toGMTString());
    }
}

// This function reads the cookie or localStorage and returns its value as a string.  
function ReadStorageString(name)
{
    delim = gDefDelim;
    if (typeof (gameDelim) != "undefined")
    {
        delim = gameDelim;
    }

    var val = "";
    if (UseStorage())
    {
        var temp = String(localStorage.getItem(name + delim));
        if (temp != "null")
        {
            val = temp;
        }
    }
    else
    {
        var fullName = name + delim + "=";
        var strings = document.cookie.split(';');
        for (var i = 0; i < strings.length; i++)
        {
            var str = strings[i];
            while (str.charAt(0) == ' ')
            {
                str = str.substring(1, str.length);
            }
                
            if (str.indexOf(fullName) == 0)
            {
                val = str.substring(fullName.length, str.length);
                break;
            }                
        }
    }
    return val;
}




//--------------------------------------------------------//
// Other functions
//
// 
function ConvertBrackets(source)
{
    var openBrackets = "[[";
    var closeBrackets = "]]";

    var index = 0;
    var start = source.indexOf(openBrackets, index);
    var end;
    while (start != -1)
    {
        end = source.indexOf(closeBrackets, index);
        if (end == -1)
        {
            break;
        }

        var fullStr = source.substring(start, end + closeBrackets.length);
        var fullfcn = source.substring(start + openBrackets.length, end);

        var fn = new Function("return " + fullfcn);
        var str = fn();
        source = source.replace(fullStr, str);      

        // Since we just replaced the [[Function()]] in the source 
        // (and the replacement could be shorter than the original)
        // we need to start the search from the start.
        // This code will break if the function returns a string 
        // with "[["
        index = 0;
        start = source.indexOf(openBrackets, index);
    }

    return source;
}
