// These are helper functions that you may find useful in writing your game.
// They're not necesssary for the basic functionality of the engine.
// You should not change this file.  Any code that you need for your game should go in _game.js

var gAchievements = "Achievements";
var gPersistsBackup = "";

var gCssSize = "cssSize";
var gUserSize = "userSize";
var gUserChose = "userChose";

var gSaveLocation = "SaveLocation";
var gSaveState = "SaveState";

var gInStats = "inStats";
var gStatsSavedPage = "StatsSavedPage";

var gDefTitle = "Adventure Creator Game";



//--------------------------------------------------------//
// Variable functions
//
// These functions are used for storing and changing 
// variables in the site.  These are the ones games 
// should call.
//
// Note that the main two functions, ReadVar and SetVar, are in engine.js

// This function adds an arbitrary amount to the variable's value.  
// If the amount isn't provided, it defaults to 1
function AddToVar(name, amount)
{
    if (typeof (amount) == "undefined")
    {
        amount = 1;
    }

    var val = ReadVar(name);
    val += amount;
    SetVar(name, val);
}

// This function subtracts an arbitrary amount from a variable's value. 
// If the value would have gone negative, it sets the value to 0.
// If the amount isn't provided, it defaults to 1
function SubFromVar(name, amount)
{
    if (typeof (amount) == "undefined")
    {
        amount = 1;
    }

    var val = ReadVar(name);
    if (val > amount)
    {
        val -= amount;
    }
    else
    {
        val = 0;
    }

    SetVar(name, val);
}


// This function adds an arbitrary amount to a variable's value if the second variable is 0
// If the amount isn't provided, it defaults to 1
function AddToVarIf0(name, checkName, amount)
{
    if (typeof (amount) == "undefined")
    {
        amount = 1;
    }

    var val = ReadVar(name);
    var check = ReadVar(checkName);
    if (check == 0)
    {
        val += amount;
        SetVar(name, val);
    }
}


// This function can replace simple "check" functions. Give it the variable and a check value, and it 
// will navigate to the link1 if the variable is greater than the check.  It wil
// navigate to link2 if the variable is less than or equal to the check.
// ie:  Choose('cinema', 0, 'beenthere.html', 'cinema1.html');
function Choose(name, check, link1, link2)
{
    var val = ReadVar(name);
    if (val > check)
    {
        GotoPage(link1);
    }
    else
    {
        GotoPage(link2);
    }

    return false;
}


// These functions are for setting, clearing, and checking the various events.
// You call them from an onclick or preload in AdvetureCreator.  The first 
// argument is the name of the variable (ie "events") and the second  
// is the name of the flag (which is a variable set to a number from 0 to 31).
function SetEvent(eventVar, name)
{
    SetVarBit(eventVar, window[name]);
}

function ClearEvent(eventVar, name)
{
    ClearVarBit(eventVar, window[name]);
}

function CheckEvent(eventVar, name)
{
    var didEvent = ReadVarBit(eventVar, window[name]);
    return didEvent;
}


// The next four functions are "bitwise."  They are used for 
// packing multiple "yes/no" variables into one variable.
// The index starts from 0.
function SetVarBit(name, index)
{
    var val = ReadVar(name);
    var mask = 1 << index;
    val = val | mask;
    SetVar(name, val);
}

function ClearVarBit(name, index)
{
    var val = ReadVar(name);
    var mask = ~(1 << index);
    val = val & mask;
    SetVar(name, val);
}

function ReadVarBit(name, index)
{
    var val = ReadVar(name);
    val = (val >> index) & 0x01;
    return val;
}

function CountVarBits(name)
{
    var val = ReadVar(name);
    var count = 0;
    while (val != 0)
    {
        count += val & 0x01;
        val >>= 1;
    }

    return count;
}

// This function sets the variable to a string value
function SetVarString(name, value)
{
    SetVar(name, value);
}

// This function reads the variable and returns its value as a string.  
function ReadVarString(name)
{
    var fullName = name + "=";
    var val = 0;
    var strings = ReadStorageString(gVariables).split(':');
    for (var i = 0; i < strings.length; i++)
    {
        var str = strings[i];
        while (str.charAt(0) == ' ')
        {
            str = str.substring(1, str.length);
        }

        if (str.indexOf(fullName) == 0)
        {
            val = str.substring(fullName.length, str.length);
            break;
        }
    }
    return val;
}

// Usually called in _begin.htm to initialize the variables
function InitVars(all)
{
    if (typeof (all) == "undefined")
    {
        all = false;
    }

    var oldSize = ReadVar(gCssSize);

    if (!all)
    {
        SavePersists();
        SetStorageString(gVariables, "");
        RestorePersists();
    }
    else
    {
        SetStorageString(gVariables, "");
    }

    for (var val in gameVars)
    {
        SetVar(gameVars[val], 0);
    }

    ChooseSize();

    var newSize = ReadVar(gCssSize);

    if (newSize != oldSize)
    {
        location.reload();
    }
}


//--------------------------------------------------------//
// Functions to translate from old school AC ones
//
// I never liked the inconsistency in AC function naming
// and capitalization.  This time I cleaned that up.
// These functions are provided because old habits die hard
//
function readVar(name)
{
    return ReadVar(name);
}

function setVar(name, value)
{
    SetVar(name, value);
}

function varPlus1(name)
{
    AddToVar(name, 1);
}

function varMinus1(name)
{
    SubFromVar(name, 1);
}

function varPlus(name, amount)
{
    AddToVar(name, amount);
}

function varMinus(name, amount)
{
    SubFromVar(name, amount);
}

function RestoreStateFromStats(delim)
{
    RestoreState(delim);
}

//--------------------------------------------------------//
// Function for putting the Score on the screen
//
function OutputScoreString(string)
{
    div = document.getElementById("scorediv");
    if (div != null)
    {
        div.innerHTML = "<p id='score'>" + string + "</p>"
    }
}


//--------------------------------------------------------//
// Functions for the stats page
//
// These functions are for going to and from the stats page.
//

// Saves the current page so that we can return to it.
function GotoStatsPage(page)
{
    SetVar(gInStats, 1);

    var curPage = GetCurrentPageName();
    SetStorageString(gStatsSavedPage, curPage);

    GotoPage(page);
}

// If overridePage is not set, returns to the page that was saved
// when we went to the stats page.
// If overridePage is set, we go there instead.
function ReturnFromStatsPage(overridePage)
{
    SetVar(gInStats, 0);
    var page;

    if (typeof (overridePage) != "undefined")
    {
        page = overridePage;
    }
    else
    {
        page = ReadStorageString(gStatsSavedPage);
    }
    
    GotoPage(page);
}


function SaveStateFromStats(delim)
{
    if (typeof (delim) == "undefined")
    {
        delim = "";
    }

    var page = ReadStorageString(gStatsSavedPage);
    SaveState(delim, page);
    ReturnFromStatsPage();
}



//--------------------------------------------------------//
// Functions for Saving and Restoring the state 
//
// 
function CheckIfSaveExists()
{
    var page = ReadStorageString(gSaveLocation);
    if (page == 0)
    {
        return false;
    }
    else
    {
        return true;
    }
}


function SaveState(delim, page)
{
    if (typeof (delim) == "undefined")
    {
        delim = "";
    }

    if (typeof (page) == "undefined")
    {
        page = GetCurrentPageName();
    }

    SetStorageString(gSaveLocation + delim, page);

    var str = "";

    for (var index in gameVars)
    {
        var from = gameVars[index];
        var fromVal = ReadVar(from);
        str += from + "=" + fromVal + ":";
    }

    // Strip off the last ":"
    str = str.slice(0, str.length - 1);

    SetStorageString(gSaveState + delim, str);

    return false;
}

function RestoreState(delim)
{
    if (typeof (delim) == "undefined")
    {
        delim = "";
    }

    var strings = ReadStorageString(gSaveState + delim).split(':');

    for (var index in strings)
    {
        var str = strings[index];
        var eqIndex = str.indexOf('=');
        if (eqIndex != -1)
        {
            var to = str.substring(0, eqIndex);
            var toVal = str.substring(eqIndex + 1, str.length);
            SetVarString(to, toVal);
        }
    }

    var page = ReadStorageString(gSaveLocation + delim);
    if (page != "")
    {
        // In case we're in the stats page, clear it
        var inStats = ReadVar(gInStats);
        if (typeof (inStats) != "undefined" && inStats != 0)
        {
            SetVar(gInStats, 0);
        }        

        GotoPage(page);
    }
}


//--------------------------------------------------------//
// Achievement functions
//
// These functions are used for storing and changing 
// achievements in the site.  These are the ones games 
// should call.
function ClearAllAchieves()
{
    SetStorageString(gAchievements, "");

    for (var val in gameAchieves)
    {
        SetVar(gameAchieves[val], 0);
    }
}


function SetAchieve(name, value)
{
    if (typeof (value) == "undefined")
    {
        value = 1;
    }

    var strings = ReadStorageString(gAchievements).split(':');
    var found = false;
    var varStr = "";

    for (var index in strings)
    {
        var str = strings[index];
        var eqIndex = str.indexOf('=');
        if (eqIndex != -1)
        {
            var to = str.substring(0, eqIndex);
            var toVal = str.substring(eqIndex + 1, str.length);
            if (to == name)
            {
                toVal = value;
                found = true;
            }
            varStr += to + "=" + toVal + ":";
        }
    }

    if (!found)
    {
        varStr += name + "=" + value;
    }
    else
    {
        // Strip off the last ":"
        varStr = varStr.slice(0, varStr.length - 1);
    }

    SetStorageString(gAchievements, varStr);
}


function ReadAchieve(name)
{
    var fullName = name + "=";
    var val = 0;
    var strings = ReadStorageString(gAchievements).split(':');
    for (var i = 0; i < strings.length; i++)
    {
        var str = strings[i];
        if (str == "")
        {
            continue;
        }

        while (str.charAt(0) == ' ')
        {
            str = str.substring(1, str.length);
        }
        if (str.indexOf(fullName) == 0)
        {
            val = parseInt(str.substring(fullName.length, str.length));
        }
        if (isNaN(val))
        {
            val = 0;
        }
    }
    return val;
}


function CountSetAchieves()
{
    var separator = "=";
    var count = 0;

    var strings = ReadStorageString(gAchievements).split(':');
    for (var i = 0; i < strings.length; i++)
    {
        var str = strings[i];
        if (str == "")
        {
            continue;
        }

        var val = 0;
        var index = 0;
        while (str.charAt(0) == ' ')
        {
            str = str.substring(1, str.length);
        }

        index = str.indexOf(separator)
        if (index != -1)
        {
            val = parseInt(str.substring(index + 1, str.length));
        }

        if (!isNaN(val) && val != 0)
        {
            count++;
        }
    }
    return count;
}


function CountMaxAchieves()
{
    var max = 0;
    if (typeof (gameAchieves) != "undefined")
    {
        max = gameAchieves.length;
    }

    return max;
}


function GotoPageIfNoAchieve(page, achieve)
{
    val = ReadAchieve(achieve);
    if (val == 0)
    {
        GotoPage(page);
    }

    return false;
}



//--------------------------------------------------------//
// Persists functions
//
// These functions are used for backing up and restoring
// the persistent variables inside initVars.  You shouldn't
// need to call these directly

function SavePersists()
{
    if (typeof (gamePersists) == "undefined")
    {
        return;
    }

    gPersistsBackup = "";
    for (var val in gamePersists)
    {
        gPersistsBackup += gamePersists[val] + "=" + ReadVar(gamePersists[val]) + ":";
    }
}

function RestorePersists()
{
    if (typeof (gamePersists) == "undefined")
    {
        return;
    }

    var strings = gPersistsBackup.split(':');

    for (var index in strings)
    {
        var str = strings[index];
        var eqIndex = str.indexOf('=');
        if (eqIndex != -1)
        {
            var to = str.substring(0, eqIndex);
            var toVal = str.substring(eqIndex + 1, str.length);
            if (to == name)
            {
                toVal = value;
            }
            SetVar(to, toVal);
        }
    }
}



//--------------------------------------------------------//
// Debug Functions
// 
// This function is used for debugging the game.
// To make use of it, go into your _game.js file and set
// "gameDoDebug" to 1. You will also have needed to set your 
// game variables into an array called gameVars.
// For example, the top lines of your _game.js would look like this:
//var gameVars = ["happy", "angry", "difficulty"];
//var gameDoDebug = 1;

function HelperShowDebug()
{
    var doDebug = 0;
    var numDebugCols = 1;
    var curCol = 0;

    if (typeof (gameDoDebug) != "undefined")
    {
        doDebug = gameDoDebug;
    }

    if (typeof (gameNumDebugCols) != "undefined")
    {
        numDebugCols = gameNumDebugCols;
    }

    if (doDebug)
    {
        var div = document.getElementById("debug");
        if (div == null)
        {
            // The page isn't set up to allow debugging
            return;
        }
        else
        {
            div.innerHTML = "";
        }

        var dbgstr = "<p class='debug'>"

        dbgstr += "Current page = " + GetCurrentPageName() + " <input type='text' id='debugNewPage' /><button onclick='GotoDebugNewPage()'>Goto Page</button>";


        if (typeof (GameChangeAllEvents) != "undefined")
        {
            dbgstr += "<br/>Modify All Events <button onclick=\"GameChangeAllEvents(true);\">set</button> <button onclick=\"GameChangeAllEvents(false);\">clear</button>";
        }

        if (typeof (GameDebugConfigureEvents) != "undefined")
        {
            dbgstr += GameDebugConfigureEvents();
        }

        dbgstr += "</p><table class='debug'><tr>";

        for (var val in gameVars)
        {
            var str = "";

            if (curCol >= numDebugCols)
            {
                str += "</tr><tr>"
                curCol = 0;
            }

            str += "<td class='debug'>" + FormatDebugVar(gameVars[val]) + "</td>";

            dbgstr += str;

            curCol++;
        }
        dbgstr += "</tr></table></div>";

        if (typeof (gamePersists) != "undefined")
        {
            curCol = 0;
            dbgstr += "</p><table class='debug'><tr>";

            for (var val in gamePersists)
            {
                var str = "";

                if (curCol >= numDebugCols)
                {
                    str += "</tr><tr>"
                    curCol = 0;
                }

                str += "<td class='debug'>" + FormatDebugVar(gamePersists[val]) + "</td>";

                dbgstr += str;

                curCol++;
            }
            dbgstr += "</tr></table></div>";
        }


        var maxAchieves = CountMaxAchieves();
        if (maxAchieves > 0)
        {
            curCol = 0;
            dbgstr += "<br/><table class='debug'><tr>";

            for (var val in gameAchieves)
            {
                var str = "";

                if (curCol >= numDebugCols)
                {
                    str += "</tr><tr>"
                    curCol = 0;
                }

                str += "<td class='debug'>" + FormatDebugAchieve(gameAchieves[val]) + "</td>";

                dbgstr += str;

                curCol++;
            }
            dbgstr += "</tr></table>";
        }

        div.innerHTML = dbgstr;
    }
}

function FormatDebugVar(name)
{
    var val = ReadVar(name);

    var str = name + " = " + val + " <button onclick=\"SubFromVar('" + name + "', 1);\">sub</button> <button onclick=\"AddToVar('" + name + "', 1);\" >add</button>";

    return str;

}

function FormatDebugAchieve(name)
{
    var val = ReadAchieve(name);

    var str = name + " = " + val + " <button onclick=\"SetAchieve('" + name + "', 0);\" >clr</button> <button onclick=\"SetAchieve('" + name + "');\" >set</button>";

    return str;

}

function GotoDebugNewPage()
{
    var page = document.getElementById("debugNewPage").value;
    if (page != "")
    {
        GotoPage(page);
    }
}


//--------------------------------------------------------//
// Function for setting the page title
//
function SetTitle()
{
    var title = gDefTitle;

    if (typeof (gameTitle) != "undefined")
    {
        title = gameTitle;
    }

    document.title = title;
}

//--------------------------------------------------------//
// Functions for setting the image size
//
// 

function SetSize(size)
{
    SetVar(gCssSize, size);
    SetVar(gUserSize, size);
    SetVar(gUserChose, 1);
    location.reload();
}

function PickSizeCss(path)
{
    var dir = "";

    if (typeof (path) != "undefined")
    {
        dir = path;
    }

    var size = ReadVar(gCssSize);
    var cssFile = "size" + size + ".css";

    document.write("<link href=\"" + dir + cssFile + "\" rel=\"stylesheet\" type=\"text/css\" />");
}


function ChooseSize()
{
    var userChose = ReadVar(gUserChose);   
    var size = 0;

    if (userChose)
    {
        size = ReadVar(gUserSize);    
    }
    else if (typeof (gameScreenWidths) != "undefined" && typeof (gameScreenHeights) != "undefined")
    {
        var max = gameScreenWidths.length;
        if (gameScreenHeights.length < max)
        {
            max = gameScreenHeights.length;
        }

        for (var cur = 0; cur < max; cur++)
        {
            if (screen.width >= gameScreenWidths[cur] && screen.height >= gameScreenHeights[cur])
            {
                size = cur;
            }
            else
            {
                break;
            }
        }
    }

    SetVar(gCssSize, size);
}


//--------------------------------------------------------//
// Functions to change the toptext
//
// These functions change the text above the picture. 
// If clear is true, it replaces the top text with str.
// If clear is false, it adds str to the top text.
function UpdateTopText(str, clear)
{
    var str2 = ConvertBrackets(str);

    var element = document.getElementById("toptext");
    if (clear)
    {
        element.innerHTML = str2;
    }
    else
    {
        element.innerHTML += str2;
    }
}

// This one just replaces the toptext with str
function SetTopText(str)
{
    UpdateTopText(str, true);
}


//--------------------------------------------------------//
// Animation Frame override.
//
// If you want to set an animation frame directly, you
// can use this function.
function SetAnimationFrame(anim, index)
{
    if (index >= anim.pictures.length)
    {
        index = anim.pictures.length - 1;
    }

    if (index < 0)
    {
        index = 0;
    }

    anim.curPicture = index;
    document.getElementById(gImageId).src = anim.pictures[anim.curPicture].src;
}


//--------------------------------------------------------//
// Animation Execute Code Function
//
// Use this function to run a function every n loops.
// It will run the passed in function after numLoops iterations
// of the animation.  
//
// This only works on pages with animations.
function AnimRunFunc(fn, numLoops)
{
    if (typeof (anim) != "undefined")
    {
        var num = anim.pictures.length;
        if (anim.pingpong)
        {
            num += num - 1;
        }
        var ms = num * anim.timeout * numLoops;
        var tId = setTimeout(fn, ms);
        return tId;
    }
}


//--------------------------------------------------------//
// Animation Advance Function
//
// Use this function to Go to the page passed in after n loops
// of the animation.  
//
// This only works on pages with animations.
function AnimAdvance(page, numLoops)
{
    if (typeof (anim) != "undefined")
    {
        var num = anim.pictures.length;
        if (anim.pingpong)
        {
            num += num - 1;
        }
        var ms = num * anim.timeout * numLoops;
        var str = "GotoPage(\"" + page + "\")";
        var tId = setTimeout(str, ms);
        return tId;
    }
}



//--------------------------------------------------------//
// Change Page After Timeout Function
//
// Use TimedAdvance in a preload to make the page change
// to another after so many milliseconds.
// For instance
// TimedAdvance("page2.htm", 5000);
// will move to page2.htm after 5 seconds.
function TimedAdvance(page, ms)
{
    var str = "GotoPage(\"" + page + "\")";
    setTimeout(str, ms);
}





//--------------------------------------------------------//
// OnMouse Starter Functions
//
// Simple functions that can be used with SetMove and
// SetClick in mouse animations.  SimpleMove assumes
// the page has an AnimHelper variable called "animHelper".
// 
// RunOnPageExit creates a function that is run when the
// user goes to another page.  This is especially helpfull
// for clearing mouse movement functions.

function SimpleMove(event)
{
    animHelper.UpdateImage(event);
}

function SimpleClick(loc)
{
    GotoPage(loc);
    return false;
}

function RunOnPageExit(fcn)
{
    pageExitFcn = new Function(fcn);
}

//--------------------------------------------------------//
// Set Mouse/Touch Functions
//
// These functions allow you to add the equivalents of
// onmousemove, onmousedown, etc in a way that works 
// for mouse, touch, and pen on browsers that support
// the pointer interface.  On incompatible browsers
// just mouse is supported.
//
// The "Set" functions add the functionality and the 
// corresponding "Clear" ones remove it.
//
// They default to working with the image, but you can
// override them to use any id.

function SetMove(moveFunction, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    if (window.PointerEvent)
    {
        element.addEventListener("pointermove", moveFunction, false);
    }
    else
    {
        element.addEventListener("mousemove", moveFunction, false);
    }
}


function ClearMove(moveFunction, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    if (window.PointerEvent)
    {
        element.removeEventListener("pointermove", moveFunction, false);
    }
    else
    {
        element.removeEventListener("mousemove", moveFunction, false);
    }
}


function SetDown(downFunction, param, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    var handler = function () { downFunction(param) };

    if (window.PointerEvent)
    {
        element.addEventListener("pointerdown", handler , false);
    }
    else
    {
        element.addEventListener("mousedown", handler , false);
    }

    return handler;
}


function SetDown2(downFunction, param1, param2, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    var handler = function () { downFunction(param1, param2) };

    if (window.PointerEvent)
    {
        element.addEventListener("pointerdown", handler, false);
    }
    else
    {
        element.addEventListener("mousedown", handler, false);
    }

    return handler;
}


function ClearDown(handler, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    if (window.PointerEvent)
    {
        element.removeEventListener("pointerdown", handler, false);
    }
    else
    {
        element.removeEventListener("mousedown", handler, false);
    }
}


function SetUp(upFunction, param, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    var handler = function () { upFunction(param) };

    if (window.PointerEvent)
    {
        element.addEventListener("pointerup", handler, false);
    }
    else
    {
        element.addEventListener("mouseup", handler, false);
    }

    return handler;
}


function SetUp2(upFunction, param1, param2, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    var handler = function () { upFunction(param1, param2) };

    if (window.PointerEvent)
    {
        element.addEventListener("pointerup", handler, false);
    }
    else
    {
        element.addEventListener("mouseup", handler, false);
    }

    return handler;
}


function ClearUp(handler, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    if (window.PointerEvent)
    {
        element.removeEventListener("pointerup", handler, false);
    }
    else
    {
        element.removeEventListener("mouseup", handler, false);
    }
}



function SetClick(clickFunction, param, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    var handler = function () { clickFunction(param) };

    element.addEventListener("click", handler, false);

    return handler;
}


function SetClick2(clickFunction, param1, param2, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    var handler = function () { clickFunction(param1, param2) };

    element.addEventListener("click", handler, false);

    return handler;
}


function ClearClick(handler, id)
{
    if (typeof (id) == "undefined")
    {
        id = gImageId;
    }

    var element = document.getElementById(id);

    element.removeEventListener("click", handler, false);
}



//--------------------------------------------------------//
// AnimHelper Object
//
// An object used to update the images in an animation based
// on mouse data. It assumes the existence of an 
// AdventureCreator anim object. It also assumes that all of 
// the images in the animation are the same size.
//
// The startSide is the side of the image the mouse starts on
// 123
// 456
// 789
// If 4 or 6, y data is ignored.  If 2 or 8, x data is ignored.
// When points come in that are closer to the startSide than 
// the initial point, the initial point is updated. 
//
// The middle startSide (5) is special.  It allows movement in 
// all directions, but requires 9 pictures. The first picture 
// needs to be the upper left corner.  The third needs to be
// the upper right corner.
//
// optRelative is an optional input.  If true, then mouse movement
// is relative to where the mouse was when the page loaded.  If
// false or not provided, mouse movement is relative to the side
// of the image.
//
// optFixedStart is an optional input.  If true, then the start
// point doesn't move.  If false or not provided, the start point
// will move up to the maximum size of the image. FixedStart only
// makes sense when Relative is true.
//
// optIncrementPercentage is an optional input.  It is a number
// that can range from 1 to 100.  If it is not provided, it defaults
// to 100.  At 100, you need to move across the entire image to get
// the full range of the animation.  At 50, you only need to move
// across half the image.  At 25, you move a quarter of the way. Etc.
// You should set optRelative and optFixedStart to true when using
// an incrementPercentage.  
function AnimHelper(startSide, optRelative, optFixedStart, optIncrementPercentage)
{
    this.numPicts = anim.pictures.length;
    this.maxPict = this.numPicts - 1;
    this.reverse = anim.reverse;
    this.increment = 1
    this.startSide = startSide;

    this.partialControl = false;
    this.currentlyAutomatic = false;

    this.firstPoint = true;

    this.fasterThanIn = 0;
    this.slowerThanIn = 0;
    this.fasterThanOut = 0;
    this.slowerThanOut = 0;
    this.inDelay = 0;
    this.outDelay = 0;
    this.failed = false;

    this.startTimeIn = new Date();
    this.startTimeOut = new Date();
    this.startTimeInDelay = new Date();
    this.startTimeOutDelay = new Date();

    this.maxOverride = this.maxPict;
    this.maxOverrideAutoIncrementAmount = 0;
    this.maxOverrideAutoIncrementFrame = 0;
    this.maxOverrideCanIncrement = false;

    this.prevIndex = 0;

    // These three variables are only used 
    // by the startSide==5 case.
    this.tileSizeX = 1;
    this.tileSizeY = 1;
    this.numTilesPerRow = 3;

    this.absolute = true;
    if (typeof (optRelative) != "undefined")
    {
        this.absolute = !optRelative;
    }

    this.fixedStart = false;
    if (typeof (optFixedStart) != "undefined")
    {
        this.fixedStart = optFixedStart;
    }

    this.incrementPercentage = 100;
    if (typeof (optIncrementPercentage) != "undefined")
    {
        this.incrementPercentage = optIncrementPercentage;
    }


    this.mouseData = new MouseData(startSide, this.fixedStart);

    this.UpdateImage = UpdateImage;
    this.SetTimes = SetTimes;
    this.SetMaxOverride = SetMaxOverride;
    this.AutomateMaxOverride = AutomateMaxOverride;
    this.RestartTiming = RestartTiming;

    this.HandleFirstPoint = HandleFirstPoint;
    this.SetIncrement = SetIncrement;
    this.UpdateFailedState = UpdateFailedState;

    this.SetPartialControl = SetPartialControl;
}


function UpdateImage(event)
{
    var imageChanged = false;

    // Workaround a bug in Firefox
    if (!event.hasOwnProperty('offsetX'))
    {
        event.offsetX = event.layerX - event.currentTarget.offsetLeft;
        event.offsetY = event.layerY - event.currentTarget.offsetTop;
    }

    //var image = document.getElementById("image");
    var mouseX = event.offsetX; //event.clientX - image.offsetLeft;
    var mouseY = event.offsetY; //event.clientY - image.offsetTop;

    this.HandleFirstPoint(mouseX, mouseY);

    var index = 0;

    if (this.startSide == 5)
    {
        var tileX = Math.floor(mouseX / this.tileSizeX);
        var tileY = Math.floor(mouseY / this.tileSizeY);
        index = tileY * this.numTilesPerRow + tileX;
    }
    else
    {
        this.mouseData.SetPoint(mouseX, mouseY);

        var dist = this.mouseData.GetDistance();
        index = Math.floor(dist / this.increment);
    }
    if (this.reverse)
    {
        index = this.maxPict - index;
    }

    // If the incrementPercentage is < 100, we can get an out of range index.  Guard against that.
    if (index > this.maxPict)
    {
        index = this.maxPict;
    }
    else if (index < 0)
    {
        index = 0;
    }


    if (index == this.maxOverrideAutoIncrementFrame && this.maxOverrideAutoIncrementAmount != 0 && this.maxOverrideCanIncrement)
    {
        this.SetMaxOverride(this.maxOverride + this.maxOverrideAutoIncrementAmount);
        this.maxOverrideCanIncrement = false;
    }
    else if (index == this.maxOverride && this.maxOverrideAutoIncrementAmount != 0)
    {
        this.maxOverrideCanIncrement = true;
    }
    else if (index > this.maxOverride)
    {
        this.failed = true;
        index = this.maxOverride;
        // The image didn't actually change
        // but the caller needs to know 
        // to check the failed state
        imageChanged = true;
    }

    if (index != anim.curPicture)
    {
        if (this.partialControl)
        {
            if (this.currentlyAutomatic)
            {
                // If we're automatic and the index is 0, the
                // user has returned the mouse to the start.
                // So it's time to go back to mouse control.
                if (index == 0)
                {
                    this.currentlyAutomatic = false;
                    clearInterval(anim.timerId);
                    imageChanged = true;
                }
                else
                {
                    imageChanged = false;
                }
            }
            else if (index >= this.maxPict)
            {
                this.currentlyAutomatic = true;
                anim.timerId = setInterval("AdvAnim(anim)", anim.timeout);
            }
            else 
            {
                imageChanged = true;
            }
        }
        else
        {
            imageChanged = true;
        }

        if (imageChanged)
        {
            this.prevIndex = anim.curPicture;
            this.UpdateFailedState(index, this.prevIndex);

            anim.curPicture = index;
            document.getElementById("image").src = anim.pictures[anim.curPicture].src;
        }
    }

    return imageChanged;
}


// The MaxOverride limits the number of frames that can play.
// For instance, if there are 10 pictures in the animation
// but you only want the user to be able to go halfway, 
// set the MaxOverride to 5.  
// To clear the override, set it to -1.
function SetMaxOverride(override)
{
    if (override < 0 || override > this.maxPict)
    {
        override = this.maxPict;
    }

    this.maxOverride = override;
}

// This will automatically add incrementAmount to the MaxOverride
// every time the frame gets to the MaxOverride and then goes
// back to incrementFrame.  To disable this functionality set
// the incrementAmount to 0.
function AutomateMaxOverride(incrementAmount, incrementFrame)
{
    this.maxOverrideAutoIncrementAmount = incrementAmount;
    this.maxOverrideAutoIncrementFrame = incrementFrame;
}


// Times are in Milliseconds.  0 means that timeout is not used.
// "In" is the time to go from frame 0 to maxPict.
// "Out" is the time to go from maxPict to 0.
// "FasterThan" means it has to take less time than that.
// "SlowerThan" means it has to take more time than that.
// "inDelay" is how long it has to stay at frame 0.
// "outDelay" is how long it has to stay at frame maxPict.
function SetTimes(fasterThanIn, slowerThanIn, inDelay, fasterThanOut, slowerThanOut, outDelay)
{
    this.fasterThanIn = fasterThanIn;
    this.slowerThanIn = slowerThanIn;
    this.inDelay = inDelay;
    this.fasterThanOut = fasterThanOut;
    this.slowerThanOut = slowerThanOut;
    this.outDelay = outDelay;
}

// Times normally start when the page loads.
// Call this function if you want to restart them.
function RestartTiming()
{
    this.startTimeIn = new Date();
    this.startTimeOut = new Date();
    this.startTimeInDelay = new Date();
    this.startTimeOutDelay = new Date();
}

// Partial Control is a special mode where you control the 
// animation for a subset of the frames and then it finishes
// the animation automatically.  Once the animation runs
// the user needs to move the mouse back to the starting point
// to control it again.
function SetPartialControl(numFrames, timeout)
{
    this.partialControl = true;
    anim.timeout = timeout;

    this.numPicts = numFrames;
    this.maxPict = this.numPicts - 1;
}

// The rest of these functions are internal to the object and 
// probably don't need to be called directly
function UpdateFailedState(cur, prev)
{
    var curTime = new Date();
    var elapsedIn = curTime - this.startTimeIn;
    var elapsedOut = curTime - this.startTimeOut;
    var elapsedInDelay = curTime - this.startTimeInDelay;
    var elapsedOutDelay = curTime - this.startTimeOutDelay;
    var failed = false;

    var max = this.maxPict;
    var pent = this.maxPict - 1;

    if (cur == 1 && prev == 0)
    {
        if (this.inDelay != 0 && elapsedInDelay < this.inDelay)
        {
            failed = true;
        }

        this.startTimeIn = curTime;
    }
    else if (cur == 0 && prev == 1)
    {
        if ((this.fasterThanOut != 0 && elapsedOut > this.fasterThanOut) ||
            (this.slowerThanOut != 0 && elapsedOut < this.slowerThanOut))
        {
            failed = true;
        }

        this.startTimeInDelay = curTime;
    }
    else if (cur == max && prev == pent)
    {
        if ((this.fasterThanIn != 0 && elapsedIn > this.fasterThanIn) ||
            (this.slowerThanIn != 0 && elapsedIn < this.slowerThanIn))
        {
            failed = true;
        }

        this.startTimeOutDelay = curTime;
    }
    else if (cur == pent && prev == max)
    {
        if (this.outDelay != 0 && elapsedOutDelay < this.outDelay)
        {
            failed = true;
        }

        this.startTimeOut = curTime;
    }

    if (failed)
    {
        this.failed = true;
    }
}


function SetIncrement()
{
    var image = document.getElementById("image");
    var width = image.width * this.incrementPercentage / 100;
    var height = image.height * this.incrementPercentage / 100;

    var side = this.startSide;

    if (side == 5)
    {
        this.numTilesPerRow = this.numPicts / 3;
        this.tileSizeX = width / (this.numPicts / 3);
        this.tileSizeY = height / (this.numPicts / 3);
    }
    else if (side == 4 || side == 6) // Horizontal movement
    {
        this.increment = width / this.numPicts;
    }
    else if (side == 2 || side == 8) // Vertical movement
    {
        this.increment = height / this.numPicts;
    }
    else // Diagonal movement
    {
        this.increment = Math.sqrt(width * width + height * height) / this.numPicts;
    }
}


function HandleFirstPoint(x, y)
{
    if (this.firstPoint)
    {
        if (this.absolute)
        {
            var image = document.getElementById("image");
            var minX = 0; //image.offsetLeft;
            var minY = 0; //image.offsetTop;
            var maxX = image.width; // + image.offsetLeft;
            var maxY = image.height; // + image.offsetTop;

            switch (this.startSide)
            {
                case 1:
                case 2:
                case 4:
                    x = minX;
                    y = minY;
                    break;

                case 3:
                case 6:
                    x = maxX;
                    y = minY;
                    break;


                case 7:
                case 8:
                    x = minX;
                    y = maxY;
                    break;

                case 9:
                    x = maxX;
                    y = maxY;
                    break;

            }
        }

        this.mouseData.SetPoint1(x, y);
        this.SetIncrement();

        this.firstPoint = false;
    }
}




function MouseData(startSide, fixedStart)
{
    this.x1 = 0;
    this.x2 = 0;
    this.y1 = 0;
    this.y2 = 0;
    this.direction = 0;
    this.distance = 0;
    this.ignoreX = false;
    this.ignoreY = false;
    this.startSide = startSide;
    this.fixedStart = fixedStart;

    this.SetPoint = SetPoint;
    this.SetPoint1 = SetPoint1;
    this.SetPoint2 = SetPoint2;
    this.GetDistance = GetDistance;
    this.SetStartSide = SetStartSide;
    this.MoveStartPointIfNecessary = MoveStartPointIfNecessary;

    this.SetStartSide(startSide);
}

// This is the function used to feed mouse data to the object.
function SetPoint(x, y)
{
    this.SetPoint2(x, y);
}

// This is the function used to see how far the mouse has
// moved from the initial point.
function GetDistance()
{
    var xDist = this.x1 - this.x2;
    var yDist = this.y1 - this.y2;

    return Math.sqrt(xDist * xDist + yDist * yDist);
}

// The rest of these functions are internal to the object and 
// probably don't need to be called directly
function SetStartSide(startSide)
{
    if (startSide < 1 || startSide > 9)
    {
        startSide = 5;
    }

    this.startSide = startSide;

    this.ignoreX = false;
    this.ignoreY = false;

    switch (startSide)
    {
        case 2:
        case 8:
            this.ignoreX = true;
            break;

        case 4:
        case 6:
            this.ignoreY = true;
            break;
    }
}



function SetPoint1(x1, y1)
{
    if (!this.ignoreX)
    {
        this.x1 = x1;
    }

    if (!this.ignoreY)
    {
        this.y1 = y1;
    }
}

function SetPoint2(x2, y2)
{
    if (!this.ignoreX)
    {
        this.x2 = x2;
    }

    if (!this.ignoreY)
    {
        this.y2 = y2;
    }

    this.MoveStartPointIfNecessary();
}

function MoveStartPointIfNecessary()
{
    if (this.fixedStart)
    {
        return;
    }

    var side = this.startSide;
    if (side == 5)
    {
        return;
    }

    if ((side <= 3 && this.y2 < this.y1) || (side >= 7 && this.y2 > this.y1))
    {
        this.y1 = this.y2;
    }

    var mod = side % 3;

    if ((mod == 1 && this.x2 < this.x1) || (mod == 0 && this.x2 > this.x1))
    {
        this.x1 = this.x2;
    }
}




